# Asserter

Lightweight permissions asserter for Ruby applications.

## Installation

Add this line to your application's Gemfile:

```ruby
gem 'asserter', git: 'https://github.com/wamland-team/asserter'
```

Use Bundler to install the dependency:

    $ bundle install

## Usage

Asserter is intended to be a lightweight and flexible way to grant permissions to
a given subject (a.k.a model/class). It imposes very few requirements on your application.

To get started add a `permissions` method to your class. This method must returns an array of permissions associated with that
subject. Once you have this method, include the `Asserter::Model` module in
your class to add the `permits?` method.

An example implementation is:

```ruby
# app/models/user.rb
class User < ActiveRecord::Base
  include Asserter::Model

  def permissions
    permissions.flat_map { |p| p[:name] }
  end
end
```

With this in place, a `Subject` can have a permission check performed, for
example:

```ruby
class PartnersController < ApplicationController
  def index
    fail('Not allowed') unless current_user.permits?('wamadmin.partners.*.index')
  end

  def current_user
    @current_user ||= User.find(session[:user_id])
  end
end
```

## Permission

A permission is made of one or more colon-separated parts that
describe the action being permitted. Each part of the permission can be
either a wildcard (`*`), or a "term" of characters `a-z A-Z 0-9 _ -`

A wildcard in any position will match a single term in that position. A wildcard
in the last position will match any number of terms in the action string.

| permission            | action                   | result |
|-----------------------|--------------------------|--------|
| `wamadmin`            | `wamadmin`               | permit |
| `wamadmin`            | `gamelles`               | deny   |
| `wamadmin.users.show` | `wamadmin.users.show`    | permit |
| `wamadmin.users.show` | `wamadmin.users.create`  | deny   |
| `*.users.index`       | `wamadmin.users.index`   | permit |
| `wamadmin.users.*`    | `wamadmin.users.destroy` | permit |
| `wamadmin.*`          | `wamadmin.users.update`  | permit |
| `*.create`            | `wamadmin.users.create`  | deny   |
| `*`                   | `wamadmin.users.create`  | permit |
| `wamadmin.users.*`    | `wamadmin.login`         | deny   |
| `wamadmin.*.*`        | `wamadmin.login`         | deny   |

## Contributing

Bug reports and pull requests are welcome on GitHub at https://github.com/wamland-team/asserter.
